package com.qs.mp.web.controller.api.admin;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qs.mp.channel.domain.ChannelOrder;
import com.qs.mp.channel.domain.ChannelOrderItem;
import com.qs.mp.channel.domain.vo.ChannelOrderVO;
import com.qs.mp.channel.service.IChannelOrderItemService;
import com.qs.mp.channel.service.IChannelOrderService;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.core.redis.RedisCache;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import lombok.AllArgsConstructor;
import ma.glasnost.orika.MapperFacade;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


@RestController
@RequestMapping("/api/v1/admin/channel/order")
@Api(tags = "渠道订单管理接口")
@AllArgsConstructor
public class ChannelOrderMgrController extends BaseApiController {

  @Autowired
  private IChannelOrderService channelOrderService;

  @Autowired
  private IChannelOrderItemService channelOrderItemService;

  @Autowired
  private MapperFacade mapperFacade;

  @Autowired
  private RedisCache redisCache;

  /**
   * 订单列表
   */
  @PostMapping("/list")
  @ApiOperation(value = "订单列表" , notes = "获取所有订单信息")
  public TableDataInfo list(@RequestBody JSONObject param) {
   // Long channelId = SecurityUtils.getLoginUser().getChannelId();
    Integer status = param.getInteger("status");
    startPage();
    LambdaQueryWrapper<ChannelOrder> queryWrapper = new LambdaQueryWrapper<ChannelOrder>();
    // queryWrapper.eq(ChannelOrder::getChannelId, channelId);
    queryWrapper.eq(null != status, ChannelOrder::getStatus, status);
    queryWrapper.orderByDesc(ChannelOrder::getCreatedTime);
    List<ChannelOrder> channelOrders = channelOrderService.list(queryWrapper);
    List<ChannelOrderVO> channelOrderVOList = mapperFacade.mapAsList(channelOrders, ChannelOrderVO.class);
    for (ChannelOrderVO channelOrderVO : channelOrderVOList) {
      channelOrderVO.setItems(channelOrderItemService.list(new LambdaQueryWrapper<ChannelOrderItem>()
          .eq(ChannelOrderItem::getOrderId, channelOrderVO.getOrderId())));
    }
    TableDataInfo rspData = getDataTable(channelOrders);
    rspData.setRows(channelOrderVOList);
    return rspData;
  }

  /**
   * 订单详情
   */
  @PostMapping("/detail")
  @ApiOperation(value = "订单详情" , notes = "在订单列表页面查看详情")
  public AjaxResult query(@RequestBody ChannelOrder order) {
    if(null == order || StringUtils.isBlank(order.getOrderId())) {
    	return error(ErrorCodeEnum.ERROR_CODE_1001);
    }    
    LambdaQueryWrapper<ChannelOrder> queryWrapper = new LambdaQueryWrapper<ChannelOrder>();
    queryWrapper.eq(ChannelOrder::getOrderId, order.getOrderId());
    
    ChannelOrder channelOrder = channelOrderService.getOne(queryWrapper);
    ChannelOrderVO channelOrderVO = mapperFacade.map(channelOrder, ChannelOrderVO.class);
    channelOrderVO.setItems(channelOrderItemService.list(new LambdaQueryWrapper<ChannelOrderItem>()
        .eq(ChannelOrderItem::getOrderId, channelOrderVO.getOrderId())));
    return AjaxResult.success(channelOrderVO);
  }

  

  /**
   * 订单发货
   */
  @PostMapping("/ship")
  @ApiOperation(value = "订单发货" , notes = "在订单发货页面提交")
  public AjaxResult ship(@RequestBody ChannelOrder order) {

    return AjaxResult.success();
  }


}
