package com.qs.mp.web.controller.api.user;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qs.mp.admin.domain.Ticket;
import com.qs.mp.admin.domain.vo.TicketListVO;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.PrizeStorageStatusEnum;
import com.qs.mp.common.enums.TicketStatusEnum;
import com.qs.mp.user.domain.UserCoin;
import com.qs.mp.user.domain.UserCoinLog;
import com.qs.mp.user.domain.UserCoupon;
import com.qs.mp.user.domain.UserPrizeStorage;
import com.qs.mp.user.domain.vo.UserCouponVO;
import com.qs.mp.user.service.IUserCoinLogService;
import com.qs.mp.user.service.IUserCoinService;
import com.qs.mp.user.service.IUserCouponService;
import com.qs.mp.user.service.IUserPrizeStorageService;
import com.qs.mp.user.service.IUserTicketOrderItemService;
import com.qs.mp.utils.SecurityUtils;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import java.util.Map;
import lombok.AllArgsConstructor;
import ma.glasnost.orika.MapperFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


@RestController
@RequestMapping("/api/v1/mp/user/mine")
@Api(tags = "我的接口")
@AllArgsConstructor
public class UserMineController extends BaseApiController {


  @Autowired
  private IUserTicketOrderItemService userTicketOrderItemService;

  @Autowired
  private IUserPrizeStorageService userPrizeStorageService;

  @Autowired
  private IUserCoinService userCoinService;

  @Autowired
  private IUserCouponService userCouponService;

  @Autowired
  private IUserCoinLogService userCoinLogService;

  @Autowired
  private MapperFacade mapperFacade;

  /**
   * 我的票包
   */
  @PostMapping("/ticket/list")
  @ApiOperation(value = "我的盲票列表", notes = "获取我的盲票列表")
  public TableDataInfo listTicket(@RequestBody Ticket param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    startPage();
    List<TicketListVO> ticketListVOS = userTicketOrderItemService.listMyTicketVO(userId,
        param.getStatus());
    return getDataTable(ticketListVOS);
  }

  /**
   * 我的优惠券
   */
  @PostMapping("/coupon/list")
  @ApiOperation(value = "我的优惠券列表", notes = "获取我的优惠券列表")
  public TableDataInfo listCoupon(@RequestBody UserCoupon param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    startPage();
    List<UserCouponVO> userCouponVOList = userCouponService.listCoupon(userId, param.getStatus());
    return getDataTable(userCouponVOList);
  }

  /**
   * 我的盲豆
   */
  @PostMapping("/coin/log/list")
  @ApiOperation(value = "我的盲豆记录", notes = "获取我的盲豆记录")
  public TableDataInfo listCoinLog(@RequestBody JSONObject param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    startPage();
    List<UserCoinLog> userCoinLogList = userCoinLogService.list(
        new LambdaQueryWrapper<UserCoinLog>()
            .eq(UserCoinLog::getUserId, userId).orderByDesc(UserCoinLog::getBizTime));
    return getDataTable(userCoinLogList);
  }

  /**
   * 我的实物奖品库
   */
  @PostMapping("/prize/list")
  @ApiOperation(value = "我的实物奖品库", notes = "获取我的实物奖品库")
  public TableDataInfo listPrize(@RequestBody JSONObject param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    startPage();
    List<UserPrizeStorage> userPrizeStorageList = userPrizeStorageService.list(
        new LambdaQueryWrapper<UserPrizeStorage>()
            .eq(UserPrizeStorage::getUserId, userId)
            .eq(UserPrizeStorage::getStatus, PrizeStorageStatusEnum.NOT_DISTRIBUTED)
            .orderByDesc(UserPrizeStorage::getCreatedTime));
    return getDataTable(userPrizeStorageList);
  }


  /**
   * 我的
   */
  @PostMapping("/init")
  @ApiOperation(value = "查看我的", notes = "查看我的")
  public AjaxResult getInfo(@RequestBody JSONObject param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    Map<String, Object> storageMap = userPrizeStorageService.getMap(
        new QueryWrapper<UserPrizeStorage>()
            .select("IFNULL(goods_num,0) as total")
            .eq("user_id", userId).eq("status", PrizeStorageStatusEnum.NOT_DISTRIBUTED));
    JSONObject result = new JSONObject();
    result.put("prizeNum", Integer.valueOf(String.valueOf(null != storageMap ? storageMap.get("total") : 0)));

    int ticketNum = userTicketOrderItemService.countMyTicket(userId, TicketStatusEnum.ACTIVATED);
    result.put("ticketNum", ticketNum);

    UserCoin userCoin = userCoinService.getById(userId);
    result.put("coinNum", null != userCoin ? userCoin.getCoin() : 0);
    return AjaxResult.success(result);
  }

}
