package com.qs.mp.admin.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qs.mp.admin.domain.Coupon;
import com.qs.mp.admin.domain.Goods;
import com.qs.mp.admin.domain.Ticket;
import com.qs.mp.admin.domain.TicketAwards;
import com.qs.mp.admin.domain.TicketAwardsPrize;
import com.qs.mp.admin.domain.TicketBox;
import com.qs.mp.admin.domain.TicketPackage;
import com.qs.mp.admin.domain.dto.TicketDrawNumDTO;
import com.qs.mp.admin.domain.param.TicketAwardsParam;
import com.qs.mp.admin.domain.param.TicketAwardsPrizeParam;
import com.qs.mp.admin.domain.param.TicketBoxCreateParam;
import com.qs.mp.admin.mapper.TicketBoxMapper;
import com.qs.mp.admin.service.ICouponService;
import com.qs.mp.admin.service.IGoodsService;
import com.qs.mp.admin.service.ITicketAwardsPrizeService;
import com.qs.mp.admin.service.ITicketAwardsService;
import com.qs.mp.admin.service.ITicketBoxSerialService;
import com.qs.mp.admin.service.ITicketBoxService;
import com.qs.mp.admin.service.ITicketPackageService;
import com.qs.mp.admin.service.ITicketService;
import com.qs.mp.common.enums.MqTopicType;
import com.qs.mp.common.enums.TicketBoxStatusEnum;
import com.qs.mp.common.enums.TicketPkgStatusEnum;
import com.qs.mp.common.enums.TicketPrizeTypeEnum;
import com.qs.mp.common.enums.TicketStatusEnum;
import com.qs.mp.common.enums.TicketTypeEnum;
import com.qs.mp.common.pulsar.PulsarClientService;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.common.utils.RSAUtil;
import com.qs.mp.system.service.id.BizIdGenerator;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import lombok.Data;
import ma.glasnost.orika.MapperFacade;
import org.apache.pulsar.client.api.PulsarClientException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

/**
 * <p>
 * 盲票组 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-02
 */
@Service
public class TicketBoxServiceImpl extends ServiceImpl<TicketBoxMapper, TicketBox> implements
    ITicketBoxService {

  protected final Logger logger = LoggerFactory.getLogger(this.getClass());

  @Autowired
  private MapperFacade mapperFacade;

  @Autowired
  private BizIdGenerator bizIdGenerator;

  @Autowired
  private ITicketAwardsService ticketAwardsService;

  @Autowired
  private ITicketAwardsPrizeService ticketAwardsPrizeService;

  @Autowired
  private IGoodsService goodsService;

  @Autowired
  private ICouponService couponService;

  @Autowired
  private ITicketBoxSerialService ticketBoxSerialService;

  @Autowired
  private ITicketService ticketService;

  @Autowired
  private ITicketPackageService ticketPackageService;

  @Autowired
  private PulsarClientService pulsarClientService;

  @Override
  @Transactional
  public boolean createTicketBox(TicketBoxCreateParam param) {
    // 创建盲票组
    TicketBox ticketBox = mapperFacade.map(param, TicketBox.class);
    ticketBox.setPkgQty(ticketBox.getQuantity() / ticketBox.getPkgUnit());
    ticketBox.setStatus(TicketBoxStatusEnum.WAIT);
    ticketBox.setBoxNo(ticketBoxSerialService.generateSerial(ticketBox.getType()));
    ticketBox.setBoxId(bizIdGenerator.newId());
    save(ticketBox);

    // 创建奖级
    List<TicketAwards> ticketAwardsList = new ArrayList<>();
    List<TicketAwardsPrize> awardsPrizeList = new ArrayList<>();
    for (TicketAwardsParam awardsParam : param.getAwardsList()) {
      TicketAwards ticketAwards = mapperFacade.map(awardsParam, TicketAwards.class);
      ticketAwards.setAwardsId(bizIdGenerator.newId());
      ticketAwards.setBoxId(ticketBox.getBoxId());
      BigDecimal hitRate = new BigDecimal(awardsParam.getQuantity()).divide(
          new BigDecimal(ticketBox.getQuantity()), 4, RoundingMode.DOWN);
      ticketAwards.setHitRate(hitRate);
      ticketAwardsList.add(ticketAwards);

      for (TicketAwardsPrizeParam prizeParam : awardsParam.getPrizeList()) {
        TicketAwardsPrize awardsPrize = mapperFacade.map(prizeParam, TicketAwardsPrize.class);
        awardsPrize.setPrizeId(bizIdGenerator.newId());
        awardsPrize.setBoxId(ticketBox.getBoxId());
        awardsPrize.setAwardsId(ticketAwards.getAwardsId());
        if (null == awardsPrize.getQuantity() || 0 == awardsPrize.getQuantity()) {
          // 页面没设置奖品的具体数量，则默认为整个奖项的数量
          awardsPrize.setQuantity(ticketAwards.getQuantity());
        }
        awardsPrize.setRemainQty(awardsPrize.getQuantity());
        awardsPrize.setCashedQty(0);
        if (prizeParam.getPrizeType() == TicketPrizeTypeEnum.GOODS) {
          Goods goods = goodsService.getById(awardsPrize.getRefId());
          awardsPrize.setTitle(goods.getTitle());
          awardsPrize.setPicUrl(goods.getPicUrl());
          awardsPrize.setValue(goods.getValue());
        } else if (prizeParam.getPrizeType() == TicketPrizeTypeEnum.COUPON) {
          Coupon coupon = couponService.getById(awardsPrize.getRefId());
          awardsPrize.setTitle(coupon.getTitle());
          awardsPrize.setPicUrl(coupon.getPicUrl());
          awardsPrize.setValue(coupon.getDiscount());
        } else {
          awardsPrize.setTitle("盲豆");
          awardsPrize.setPicUrl("md.jpeg");
        }
        awardsPrizeList.add(awardsPrize);
      }
    }
    ticketAwardsService.saveBatch(ticketAwardsList);
    ticketAwardsPrizeService.saveBatch(awardsPrizeList);

    try {
      pulsarClientService.producer(MqTopicType.ticket_generate, ticketBox.getBoxId());
    } catch (PulsarClientException e) {
      LogUtil.error(logger, e, "盲票组保存成功，发送异步消息失败. {0}", JSONObject.toJSONString(ticketBox));
    }
    return true;
  }

  @Override
  @Transactional
  public void removeTicketBox(String boxId) {
    removeById(boxId);
    ticketPackageService.remove(new LambdaQueryWrapper<TicketPackage>().eq(TicketPackage::getBoxId, boxId));
    ticketService.remove(new LambdaQueryWrapper<Ticket>().eq(Ticket::getBoxId, boxId));
    ticketAwardsService.remove(new LambdaQueryWrapper<TicketAwards>().eq(TicketAwards::getBoxId, boxId));
    ticketAwardsPrizeService.remove(new LambdaQueryWrapper<TicketAwardsPrize>().eq(TicketAwardsPrize::getBoxId, boxId));
  }

  @Override
  @Transactional
  public void generateTicket(String boxId) {
    TicketBox ticketBox = getById(boxId);
    Assert.isTrue(ticketBox.getStatus() == TicketBoxStatusEnum.DOING,
        "盲票生成时，票组状态不是出票中，boxId=" + boxId);
    List<TicketAwards> ticketAwardsList = ticketAwardsService.list(
        new LambdaQueryWrapper<TicketAwards>()
            .eq(TicketAwards::getBoxId, ticketBox.getBoxId()));
    // 分包
    int pkgNum = ticketBox.getPkgQty();
    int pkgUnit = ticketBox.getPkgUnit();

    Map<Integer, List<PkgAwards>> pkgAwardsMap = generatePkgAwards(
        ticketBox, ticketAwardsList, pkgNum, pkgUnit);

    // 生成票包记录和盲票记录
    int pkgCnt = 0;
    int ticketCnt = 0;
    List<TicketPackage> ticketPackageList = new ArrayList<>();
    for (Integer key : pkgAwardsMap.keySet()) {
      pkgCnt += 1;
      TicketPackage ticketPackage = new TicketPackage();
      ticketPackage.setPkgId(bizIdGenerator.newId());
      ticketPackage.setBoxId(ticketBox.getBoxId());
      ticketPackage.setPkgNo(ticketBox.getBoxNo() + "-" + String.format("%1$04d", pkgCnt));
      ticketPackage.setStartSn(
          ticketPackage.getPkgNo() + "-" + String.format("%1$07d", ticketCnt + 1));
      ticketPackage.setEndSn(
          ticketPackage.getPkgNo() + "-" + String.format("%1$07d", ticketCnt + pkgUnit));
      ticketPackage.setStatus(TicketPkgStatusEnum.FOR_SALE);
      ticketPackageList.add(ticketPackage);

      List<PkgAwards> pkgAwardsList = pkgAwardsMap.get(key);
      LogUtil.debug(logger, "第{0}包盲票奖项数量为{1}", pkgCnt, pkgAwardsList);
      List<Ticket> ticketList = new ArrayList<>();
      for (int j = 1; j <= pkgUnit; j++) {
        ticketCnt += 1;
        Iterator<PkgAwards> iterator = pkgAwardsList.iterator();
        while (iterator.hasNext()) {
          PkgAwards next = iterator.next();
          if (next.getQuantity() == 0) {
            iterator.remove();
          }
        }
        LogUtil.debug(logger, "开始生成第{0}包、第{1}盲票", pkgCnt, j);
        int random = getPrizeIndex(pkgAwardsList);
        PkgAwards pkgAwards = pkgAwardsList.get(random);
        pkgAwards.setQuantity(pkgAwards.getQuantity() - 1);

        Ticket ticket = new Ticket();
        ticket.setTicketId(bizIdGenerator.newIdWithUidSharding(ticketBox.getBoxNo()));
        ticket.setBoxId(ticketBox.getBoxId());
        ticket.setPkgId(ticketPackage.getPkgId());
        ticket.setTitle(ticketBox.getTitle());
        ticket.setSerialNo(ticketPackage.getPkgNo() + "-" + String.format("%1$07d", ticketCnt));
        ticket.setFacePrice(ticketBox.getFacePrice());
        int luckyNum = new Random().nextInt(99) + 1;
        ticket.setCipherLuckyNum(RSAUtil.encrypt(String.valueOf(luckyNum)));
        List<TicketDrawNumDTO> drawNumDTOList = new ArrayList<>();
        drawNumDTOList.add(new TicketDrawNumDTO(pkgAwards.getName(), luckyNum));
        // 再生成17个随机数字
        List<Integer> randomNumList = getRandomList(Arrays.asList(luckyNum), 17, 99);
        for (Integer drawNum : randomNumList) {
          int awardsNum = new Random().nextInt(ticketAwardsList.size());
          drawNumDTOList.add(
              new TicketDrawNumDTO(ticketAwardsList.get(awardsNum).getName(), drawNum));
        }
        Collections.shuffle(drawNumDTOList);

        ticket.setDrawNum(JSONObject.toJSONString(drawNumDTOList));
        ticket.setIsPhysical(1);
        ticket.setStatus(TicketStatusEnum.NOT_PAY);
        ticketList.add(ticket);
      }
      ticketService.saveBatch(ticketList);
    }
    ticketPackageService.saveBatch(ticketPackageList);

    boolean rst = update(new LambdaUpdateWrapper<TicketBox>().set(TicketBox::getStatus, TicketBoxStatusEnum.DONE)
        .eq(TicketBox::getBoxId, boxId).eq(TicketBox::getStatus, TicketBoxStatusEnum.DOING));
    Assert.isTrue(rst, "盲票生成完，更新盲票组状态失败。boxId:{0}" + ticketBox.getBoxId());
  }

  private Map<Integer, List<PkgAwards>> generatePkgAwards(TicketBox ticketBox,
      List<TicketAwards> ticketAwardsList, int pkgNum, int pkgUnit) {
    // 1.把各奖级的中奖数量分摊到每包上
    Map<Integer, List<PkgAwards>> pkgAwardsMap = new HashMap<>();
    for (int m = 1; m <= pkgNum; m++) {
      pkgAwardsMap.put(m, new ArrayList<PkgAwards>());
    }
    List<Integer> excludePkgList = new ArrayList<>();
    for (int k = 0; k < ticketAwardsList.size(); k++) {
      TicketAwards ticketAwards = ticketAwardsList.get(k);
      if (ticketAwards.getQuantity() < pkgNum) {
        // 奖级数量少于包数的，随机不重复分配，随机数从1开始
        int totalNone = pkgNum - ticketAwards.getQuantity(); // 轮空数
        int moreExInt = excludePkgList.size() - totalNone; // 本轮要排除的数 - 轮空数
        if (moreExInt > 0) {
          // 多出来的数，从末尾开始删除
          for (int l = 0; l < moreExInt; l++) {
            excludePkgList.remove(excludePkgList.size() - 1);
          }
        }
        List<Integer> randomList = getRandomList(excludePkgList, ticketAwards.getQuantity(),
            pkgNum);
        excludePkgList.addAll(randomList);
        LogUtil.debug(logger, "随机分配到的包序号为:{0}", JSONObject.toJSONString(randomList));
        for (Integer pkgId : randomList) {
          pkgAwardsMap.get(pkgId).add(
              new PkgAwards(ticketAwards.getAwardsId(), ticketAwards.getName(),
                  ticketAwards.getSort(), 1));
        }
      } else {
        // 奖级数量大于包数的，平均分配，四舍五入
        int remainQty = ticketAwards.getQuantity();
        for (int i = 1; i <= pkgNum; i++) {
          int quantity;
          if (k == ticketAwardsList.size() - 1) {
            // 最后一个奖项直接分配奖级数量差额即可
            int hasQty = pkgAwardsMap.get(i).stream().mapToInt(PkgAwards::getQuantity).sum();
            quantity = pkgUnit - hasQty;
          } else {
            if (i == pkgNum) {
              quantity = remainQty;
            } else {
              quantity = new BigDecimal(remainQty).divide(new BigDecimal(pkgNum - i + 1), 0,
                  RoundingMode.HALF_UP).intValue();
            }
          }
          remainQty -= quantity;
          Assert.isTrue(remainQty >= 0, "剩余奖级数量不足分配。boxId:" + ticketBox.getBoxId());
          pkgAwardsMap.get(i).add(
              new PkgAwards(ticketAwards.getAwardsId(), ticketAwards.getName(),
                  ticketAwards.getSort(), quantity));
        }
      }
      LogUtil.debug(logger, "奖级{0}分包结果:{1}",
          k, JSONObject.toJSONString(pkgAwardsMap));
    }
    return pkgAwardsMap;
  }

  /**
   * 根据Math.random()产生一个double型的随机数，判断每个奖品出现的概率
   *
   * @param prizes
   * @return random：奖品列表prizes中的序列（prizes中的第random个就是抽中的奖品）
   */
  public int getPrizeIndex(List<PkgAwards> prizes) {
    DecimalFormat df = new DecimalFormat("######0.00");
    int random = -1;

    //计算总权重
    double sumWeight = 0;
    for (PkgAwards p : prizes) {
      sumWeight += p.getQuantity();
    }
    LogUtil.info(logger, "sumWeight:{0}", sumWeight);
    //产生随机数
    double randomNumber;
    randomNumber = Math.random();
    LogUtil.debug(logger, "随机数：" + randomNumber);
    //根据随机数在所有奖品分布的区域并确定所抽奖品
    double d1 = 0;
    double d2 = 0;
    for (int i = 0; i < prizes.size(); i++) {
      d2 += Double.parseDouble(String.valueOf(prizes.get(i).getQuantity())) / sumWeight;
      if (i == 0) {
        d1 = 0;
      } else {
        d1 += Double.parseDouble(String.valueOf(prizes.get(i - 1).getQuantity())) / sumWeight;
      }
      LogUtil.debug(logger, "prize：{0}，区间 d1:{1}, d2:{2}",
          JSONObject.toJSONString(prizes.get(i)), d1, d2);
      if (randomNumber > d1 && randomNumber <= d2) {
        random = i;
        break;
      }
    }
    LogUtil.debug(logger, "抽中序号:{0}", random);
    return random;
  }

  /**
   * 从total中随机取n个不重复的数，范围[1, total]
   *
   * @param n
   * @param total
   * @return
   */
  private List<Integer> getRandomList(List<Integer> excludeList, int n, int total) {
    List<Integer> randomList = new ArrayList<>();
    Random rand = new Random();
    boolean[] bool = new boolean[total];
    for (Integer exInt : excludeList) {
      bool[exInt - 1] = true;
    }
    int randInt = 0;
    for (int i = 0; i < n; i++) {
      do {
        randInt = rand.nextInt(total);
      } while (bool[randInt]);
      bool[randInt] = true;
      randomList.add(randInt + 1);
    }
    return randomList;
  }

  @Data
  private static class PkgAwards {

    private String awardsId;

    /**
     * 奖项名
     */
    private String name;

    /**
     * 奖项顺序
     */
    private Integer sort;

    /**
     * 奖项数量
     */
    private Integer quantity;

    PkgAwards(String awardsId, String name, Integer sort, Integer quantity) {
      this.awardsId = awardsId;
      this.name = name;
      this.sort = sort;
      this.quantity = quantity;
    }
  }

  public static void main(String[] args) {
    List<PkgAwards> prizes = new ArrayList<>();
    prizes.add(new PkgAwards("1", "一等奖", 1, 1));
    prizes.add(new PkgAwards("2", "二等奖", 2, 2));
    prizes.add(new PkgAwards("3", "三等奖", 3, 3));
    prizes.add(new PkgAwards("4", "四等奖", 4, 4));
    TicketBoxServiceImpl boxService = new TicketBoxServiceImpl();
    System.out.println(boxService.getRandomList(new ArrayList<>(), 10, 10));
    /*for (int i = 0 ; i < 10; i ++ ) {
      int random = boxService.getPrizeIndex(prizes);
      System.out.println(random);
    }*/
  }
}
