package com.qs.mp.channel.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qs.mp.admin.domain.TicketBox;
import com.qs.mp.admin.service.ITicketBoxService;
import com.qs.mp.user.domain.UserAddr;
import com.qs.mp.channel.domain.ChannelOrder;
import com.qs.mp.channel.domain.ChannelOrderItem;
import com.qs.mp.channel.domain.vo.ChannelCartVO;
import com.qs.mp.channel.domain.vo.ChannelOrderSettleVO;
import com.qs.mp.channel.mapper.ChannelOrderMapper;
import com.qs.mp.channel.service.IChannelCartService;
import com.qs.mp.channel.service.IChannelOrderItemService;
import com.qs.mp.channel.service.IChannelOrderService;
import com.qs.mp.common.enums.ChannelOrderStatusEnum;
import com.qs.mp.common.enums.TicketBoxStatusEnum;
import com.qs.mp.common.exception.ServiceException;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.pay.domain.PayOrder;
import com.qs.mp.system.service.id.BizIdGenerator;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 经销商盲票采购订单 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-02
 */
@Service
public class ChannelOrderServiceImpl extends
    ServiceImpl<ChannelOrderMapper, ChannelOrder> implements IChannelOrderService {

  protected final Logger logger = LoggerFactory.getLogger(this.getClass().getSimpleName());

  @Autowired
  private BizIdGenerator bizIdGenerator;

  @Autowired
  private ITicketBoxService ticketBoxService;

  @Autowired
  private IChannelOrderItemService channelOrderItemService;

  @Autowired
  private IChannelOrderService channelOrderService;

  @Autowired
  private IChannelCartService channelCartService;

  @Override
  @Transactional
  public String submitOrder(Long channelId, String memo, ChannelOrderSettleVO orderSettleVO) {
    UserAddr addr = orderSettleVO.getAddr();
    String orderId = bizIdGenerator.newId();
    int prodNum = 0;
    StringBuilder title = new StringBuilder();
    // 创建订单明细
    List<ChannelOrderItem> orderItemList = new ArrayList<>();
    for (ChannelCartVO channelCartVO : orderSettleVO.getItems()) {
      TicketBox ticketBox = ticketBoxService.getById(channelCartVO.getBoxId());
      if (ticketBox.getSalePkgQty() + channelCartVO.getOrderNum() > ticketBox.getPkgQty()) {
        throw new ServiceException(ticketBox.getTitle() + "库存不足，请重新下单");
      }
      ChannelOrderItem item = new ChannelOrderItem();
      item.setOrderId(orderId);
      item.setChannelId(channelId);
      item.setTitle(ticketBox.getTitle());
      item.setPicUrl(ticketBox.getPicUrl());
      item.setBoxId(channelCartVO.getBoxId());
      item.setProperties(ticketBox.getProperties());
      item.setPkgUnit(ticketBox.getPkgUnit());
      item.setFacePrice(ticketBox.getFacePrice());
      item.setOrderNum(channelCartVO.getOrderNum());
      item.setPkgSalePrice(ticketBox.getPkgSalePrice());
      orderItemList.add(item);
      // 累计商品数，拼接标题
      prodNum += channelCartVO.getOrderNum();
      title.append(ticketBox.getTitle()).append(",");
      // 更新盲票组的已销售数和状态，乐观锁
      boolean rst = ticketBoxService.update(new LambdaUpdateWrapper<TicketBox>()
          .set(ticketBox.getSalePkgQty() + channelCartVO.getOrderNum() == ticketBox.getPkgQty(),
              TicketBox::getStatus,
              TicketBoxStatusEnum.PUT_OFF)
          .set(TicketBox::getSalePkgQty, ticketBox.getSalePkgQty() + channelCartVO.getOrderNum())
          .eq(TicketBox::getSalePkgQty, ticketBox.getSalePkgQty())
          .eq(TicketBox::getBoxId, ticketBox.getBoxId()));
      if (!rst) {
        LogUtil.error(logger, "渠道采购票组订单创建时，更新盲票组销量失败。boxId:{0}, saleQty:{1}",
            ticketBox.getBoxId(), ticketBox.getSaleQty());
        throw new ServiceException("渠道采购票组订单创建时，更新盲票组销量失败");
      }
      // 删除购物车
      if (null != channelCartVO.getCartId() && 0 != channelCartVO.getCartId()) {
        channelCartService.removeById(channelCartVO.getCartId());
      }
    }

    // 创建订单
    ChannelOrder channelOrder = new ChannelOrder();
    channelOrder.setOrderId(orderId);
    channelOrder.setChannelId(channelId);
    channelOrder.setTitle(title.substring(0, title.length() - 1));
    channelOrder.setOrderAmt(orderSettleVO.getOrderAmt());
    channelOrder.setDiscountAmt(orderSettleVO.getDiscountAmt());
    channelOrder.setFreightAmt(orderSettleVO.getFreightAmt());
    channelOrder.setPayAmt(orderSettleVO.getPayAmt());
    channelOrder.setStatus(ChannelOrderStatusEnum.NOT_PAY);
    channelOrder.setPkgNum(orderSettleVO.getPkgNum());
    channelOrder.setMemo(memo);
    channelOrder.setReceiver(addr.getReceiver());
    channelOrder.setTel(addr.getMobile());
    channelOrder.setProvince(addr.getProvince());
    channelOrder.setCity(addr.getCity());
    channelOrder.setArea(addr.getArea());
    channelOrder.setAddress(addr.getAddr());

    channelOrderService.save(channelOrder);
    channelOrderItemService.saveBatch(orderItemList);
    return channelOrder.getOrderId();
  }

  @Override
  @Transactional
  public boolean cancelOrder(Long channelId, String orderId) {
    ChannelOrder channelOrder = channelOrderService.getOne(new LambdaQueryWrapper<ChannelOrder>()
        .eq(ChannelOrder::getChannelId, channelId).eq(ChannelOrder::getOrderId, orderId));
    if (channelOrder.getStatus() != ChannelOrderStatusEnum.NOT_PAY) {
      throw new ServiceException("订单状态异常，不能取消");
    }

    List<ChannelOrderItem> orderItemList = channelOrderItemService.list(
        new LambdaQueryWrapper<ChannelOrderItem>()
            .eq(ChannelOrderItem::getOrderId, orderId));
    for (ChannelOrderItem orderItem : orderItemList) {
      TicketBox ticketBox = ticketBoxService.getById(orderItem.getBoxId());
      // 恢复盲票组的已销售数和状态，乐观锁
      boolean rst = ticketBoxService.update(new LambdaUpdateWrapper<TicketBox>()
          .set(
              ticketBox.getStatus() == TicketBoxStatusEnum.PUT_OFF && ticketBox.getManualOff() != 1,
              TicketBox::getStatus,
              TicketBoxStatusEnum.PUT_ON)
          .set(TicketBox::getSalePkgQty, ticketBox.getSalePkgQty() - orderItem.getOrderNum())
          .eq(TicketBox::getSalePkgQty, ticketBox.getSalePkgQty())
          .eq(TicketBox::getBoxId, ticketBox.getBoxId()));
      if (!rst) {
        LogUtil.error(logger, "渠道采购票组订单取消时，更新盲票组销量失败。boxId:{0}, saleQty:{1}",
            ticketBox.getBoxId(), ticketBox.getSaleQty());
        throw new ServiceException("渠道采购票组订单取消时，更新盲票组销量失败");
      }
    }
    boolean rst = channelOrderService.update(
        new LambdaUpdateWrapper<ChannelOrder>().set(ChannelOrder::getStatus,
                ChannelOrderStatusEnum.CANCELED)
            .eq(ChannelOrder::getOrderId, orderId)
            .eq(ChannelOrder::getStatus, ChannelOrderStatusEnum.NOT_PAY));
    if (!rst) {
      LogUtil.error(logger, "渠道采购票组订单取消时，更新订单状态失败。orderId:{0}", orderId);
      throw new ServiceException("渠道采购票组订单取消时，更新订单状态失败");
    }
    return true;
  }

  @Override
  public void paySuccess(PayOrder payOrder) {
    ChannelOrder channelOrder = getById(payOrder.getBizId());
    if (null == channelOrder || channelOrder.getStatus() != ChannelOrderStatusEnum.NOT_PAY) {
      LogUtil.error(logger, "支付回调渠道订单处理，订单状态异常，不是待支付。payOrder:{0}, channelOrder:{1}",
          JSONObject.toJSONString(payOrder), JSONObject.toJSONString(channelOrder));
      throw new ServiceException("支付回调渠道订单处理，订单状态异常，不是待支付。orderId:" + payOrder.getBizId());
    }
    boolean rst = update(
        new LambdaUpdateWrapper<ChannelOrder>().set(ChannelOrder::getStatus,
                ChannelOrderStatusEnum.NOT_DELIVER)
            .eq(ChannelOrder::getOrderId, channelOrder.getOrderId())
            .eq(ChannelOrder::getStatus, ChannelOrderStatusEnum.NOT_PAY));
    if (!rst) {
      LogUtil.error(logger, "渠道采购票组订单支付成功时，更新订单状态失败。orderId:{0}",
          channelOrder.getOrderId());
      throw new ServiceException("渠道采购票组订单支付成功时，更新订单状态失败");
    }
  }
}
