package com.qs.mp.channel.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.domain.ChannelBankCard;
import com.qs.mp.channel.domain.ChannelMoneyLog;
import com.qs.mp.channel.domain.ChannelWithdraw;
import com.qs.mp.channel.mapper.ChannelWithdrawMapper;
import com.qs.mp.channel.service.IChannelBankCardService;
import com.qs.mp.channel.service.IChannelMoneyLogService;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.channel.service.IChannelWithdrawService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qs.mp.common.enums.ChannelMoneyEnum;
import java.util.Date;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 渠道提现表 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-04
 */
@Service
public class ChannelWithdrawServiceImpl extends ServiceImpl<ChannelWithdrawMapper, ChannelWithdraw> implements IChannelWithdrawService {

  @Autowired
  private IChannelBankCardService channelBankCardService;

  @Autowired
  private IChannelService channelService;

  @Autowired
  private IChannelMoneyLogService channelMoneyLogService;

  @Override
  @Transactional
  public boolean apply(Channel channel, ChannelWithdraw channelWithdraw) {

    // 更新用户的银行卡信息
    ChannelBankCard bankCard = channelBankCardService.getOne(new LambdaQueryWrapper<ChannelBankCard>()
        .eq(ChannelBankCard::getChannelId, channel.getChannelId()));
    if (null == bankCard) {
      bankCard = new ChannelBankCard();
      bankCard.setChannelId(channel.getChannelId());
      bankCard.setCardNo(channelWithdraw.getCardNo());
      bankCard.setBankName(channelWithdraw.getBankName());
      bankCard.setBranchName(channelWithdraw.getBranchName());
      bankCard.setUserName(channelWithdraw.getUserName());
      channelBankCardService.save(bankCard);
    } else {
      bankCard.setCardNo(channelWithdraw.getCardNo());
      bankCard.setBankName(channelWithdraw.getBankName());
      bankCard.setBranchName(channelWithdraw.getBranchName());
      bankCard.setUserName(channelWithdraw.getUserName());
      channelBankCardService.updateById(bankCard);
    }
    boolean rst = save(channelWithdraw);

    ChannelMoneyLog moneyLog = new ChannelMoneyLog();
    moneyLog.setChannelId(channel.getChannelId());
    moneyLog.setType(ChannelMoneyEnum.WITHDRAW);
    moneyLog.setLogMoney(channelWithdraw.getMoney());
    moneyLog.setBizTime(new Date());
    moneyLog.setRefId(String.valueOf(channelWithdraw.getId()));
    moneyLog.setLogText("银行卡尾号 " + channelWithdraw.getCardNo().substring(channelWithdraw.getCardNo().length() - 4));
    channelMoneyLogService.changeMoney(moneyLog);

    if (channelWithdraw.getChargeAmt() > 0) {
      // 手续费
      ChannelMoneyLog feeMoneyLog = new ChannelMoneyLog();
      feeMoneyLog.setChannelId(channel.getChannelId());
      feeMoneyLog.setType(ChannelMoneyEnum.WITHDRAW_FEE);
      feeMoneyLog.setLogMoney(channelWithdraw.getChargeAmt());
      feeMoneyLog.setBizTime(new Date());
      feeMoneyLog.setRefId(String.valueOf(channelWithdraw.getId()));
      channelMoneyLogService.changeMoney(feeMoneyLog);
    }
    // 创建提现记录
    return rst;
  }
}
